global isr_0                 ; genericka obsluha preruseni c.1
global isr_1                 ; genericka obsluha preruseni c.2
global isr_keyb              ; obsluha preruseni pri stisku klavesy
global isr_timer             ; obsluha preruseni pri aktivaci casovace
global isr_syscall           ; obsluha systemovych volani
global isr_pagefault         ; obsluha vypadku stranky
global irq_load_idt          ; inicializace IDT

extern irq_handler_generic   ; univerzalni rutina volana pri vzniku preruseni
extern irq_handler_keyb      ; rutina volana pri vzniku preruseni vyvolanem klavesnici 
extern irq_handler_timer     ; rutina volana pri vzniku preruseni vyvolanem casovacem 
extern syscall_handler       ; rutina obsluhujici systemova volani
extern vm_pagefault_handler  ; rutina zavolana pri vypadku stranky

extern task_running          ; struktura obsahujici informace o aktualne bezicim procesu
extern task_preempt          ; funkce, ktera je zavolana pri preemptivnim prepnuti procesu

PIC1_CMD   equ 0x20          ; port ridiciho registru PIC1
PIC1_DATA  equ 0x21          ; datovy port PIC1
PIC2_CMD   equ 0xa0          ; port ridiciho registr PIC2
PIC2_DATA  equ 0xa1          ; datovy port PIC2


section .text
isr_0:                        
    pushad                   ; ulozi na zasobnik vsechny registry

    push dword 0             ; zavola funkci irq_handler_generic s argumentem 0
    call irq_handler_generic
    add esp, 4               ; odstrani argument ze zasobniku

    popad                    ; obnovi ulozene registry
    iret                     ; provede navrat z obsluhy preruseni

isr_1:
    pushad

    push dword 1
    call irq_handler_generic
    add esp, 4

    popad
    iret

isr_keyb:
    pushad                   ; ulozi obsah vsech registru 

    in al, 0x60              ; precte dany scan code
    movzx eax, al
    push eax                 ; preda dany scan code do procedury irq_handler_generic
    call irq_handler_keyb
    add esp, 4

    mov al, 0x20             ; potvrdi zpracovani preruseni
    out PIC1_CMD, al

    popad                    ; obnovi registry
    iret                     ; prevede navrat z obsluhy preruseni

isr_syscall:
    push edi                 ; prevede argumenty preruseni na konvenci CDECL
    push esi
    push edx                 
    push ecx
    push ebx
    push eax
    mov eax, [task_running]
    mov dword [eax + 4], esp 

    call syscall_handler    ; zavola univerzalni funkci pro obsluha sys. volani
    
    add esp, 4
    pop ebx
    pop ecx
    pop edx
    pop esi
    pop edi
    iret 

isr_pagefault:
    pushad
    mov ecx, [esp + 32]     ; nacte ze zasobniku chybove slovo (musi preskocit registry ulozene pushad)
    mov eax, cr2            ; nacte adresu, kde doslo k vypadku

    push ecx                ; a preda je funkci, ktera obstara obsluhu 
    push eax                
    call vm_pagefault_handler
    add esp, 8

    popad
    add esp, 4              ; odstrani chybovy kod

    iret

isr_timer:
    pushad                   ; ulozi obsah vsech registru 

    mov eax, [task_running]  ; ulozi obsah esp, do TCB aktualniho procesu
    mov dword [eax + 4], esp 

    mov al, 0x20             ; potvrdi zpracovani preruseni
    out PIC1_CMD, al         ; ale protoze jsme v obsluze preruseni, nejde dalsi preruseni vyvolat

    call task_preempt        ; vola prepnuti procesu

;   popad                    ; nemelo by se to sem nikdy dostat 
;   iret                     ; provedeno pri obnoveni vlakna


;
; pomocna funkce pro nacteni IDT
; 
; argument #1 - pocet zaznamu
; argument #2 - umisteni tabulky
;
irq_load_idt:
    push ebp
    mov ebp, esp

    movzx eax, byte [ebp + 8]     ; pocet zaznamu
    shl eax, 3
    dec eax                       ; prepocet na velikost tabulky

    mov ecx, [ebp + 12]           ; tabulka

    mov [idt_desc], eax
    mov [idt_desc + 2], ecx
    
    lidt [idt_desc]               ; nastavi idt
    sti                           ; povoli preruseni (vypnute pri inicializaci chraneneho rezimu)
    
    pop ebp
    ret

;
; datova cast
;
section .bss

; descriptor IDT
idt_desc:
    resw 1 ; table size
    resd 1 ; location of entries
