/*
 * Vstupni bod jadra operacniho systemu.
 *
 * Tento zdrojovy kod by mel mit prave jednu funkci, ktera je zavolana
 * po prepnuti do 32bitoveho rezimu. (Pokud by zdrojovy soubor obsahoval
 * dalsi funkce, muze je prekladac preskladat a dostaneme nezadouci chovani.)
 */


#include "irq/irq.h"

#include "mem/alloc.h"
#include "mem/pg_alloc.h"
#include "mem/vm.h"

#include "dev/term.h"
#include "dev/keyb.h"
#include "dev/blk_dev.h"

#include "util/io.h"
#include "util/util.h"
#include "util/string.h"

#include "fs/mufs.h"
#include "tasks/readelf.h"
#include "tasks/tasks.h"

#define KERNEL_MEM_PAGES	(256)

struct pg_zone kernel_mem_zone;
struct page kernel_mem_pages[KERNEL_MEM_PAGES];

struct kmem_cache mem_ata_drives;
struct mufs root_fs;

extern struct term term;
extern struct blk_dev *blk_devs[];
extern unsigned int blk_devs_cnt;
extern struct task *task_running;

unsigned char buf[4096];

void load_tss();
#define PG_PRESENT (0x01)
#define PG_RW      (0x02)
#define PG_USER    (0x04)

void kernel_main()
{
	load_tss();
	term_init();
	kprint_color(3, 0);
	pg_zone_init(&kernel_mem_zone, KERNEL_MEM_PAGES, kernel_mem_pages, (unsigned char *) 0x00100000);
	kmem_init();
	vm_init(64 * 1024 * 1024);

	/* nastavi pocatecni rozlozeni pameti */
	uint32_t *initial_pd = vm_create_initial_page_directory();

	vm_set_page_directory(initial_pd);
	vm_enable_paging();

	keyb_init();
	task_init();
	irq_init();


	kprint_color(2, 0);
	kprint("starting...\r\n");

	blk_dev_detect();
	if (blk_devs_cnt > 1) {
		kprint("initilizing file system: ");

		int status = mufs_open(&root_fs, blk_devs[1]);
		if (status) {
			kprint("failed\r\n");
		} else {
			kprint("success\r\n");
		}

		/* povoli preruseni z casovace */
		pic_enable_irq(PIC_MASTER, 0x01);
// varianta 3
		jumpto_scheduler();	
	}
	
	while (1) {
	}
}

