#include "../fs/mufs.h"
#include "readelf.h"

#define EHDR_ASSERT(ehdr, field, expected) \
	do { \
		if ((ehdr)->field != (expected)) return EINVALID_FILE; \
	} while (0)
	
/**
 * Nacte hlavicku objektoveho souboru.
 */
int readelf_file_header(struct mufs_fd *f, Elf32_Ehdr *ehdr)
{
	int size = mufs_file_read(f, (unsigned char *) ehdr, sizeof(Elf32_Ehdr));
	
	if (size != sizeof(Elf32_Ehdr)) return EIO_ERR;
	EHDR_ASSERT(ehdr, e_ident[EI_MAG0],    ELFMAG0);
	EHDR_ASSERT(ehdr, e_ident[EI_MAG1],    ELFMAG1);
	EHDR_ASSERT(ehdr, e_ident[EI_MAG2],    ELFMAG2);
	EHDR_ASSERT(ehdr, e_ident[EI_MAG3],    ELFMAG3);
	EHDR_ASSERT(ehdr, e_ident[EI_CLASS],   ELFCLASS32);
	EHDR_ASSERT(ehdr, e_ident[EI_DATA],    ELFDATA2LSB);
	EHDR_ASSERT(ehdr, e_ident[EI_VERSION], 1);
	EHDR_ASSERT(ehdr, e_type,    ET_EXEC);
	EHDR_ASSERT(ehdr, e_machine, EM_386);
	EHDR_ASSERT(ehdr, e_version, EV_CURRENT);

	return EOK;
}

/**
 * Nacte "program header" s danym indexem
 */
int readelf_program_header(struct mufs_fd *f, int index, Elf32_Ehdr *ehdr, Elf32_Phdr *phdr)
{
	int size = ehdr->e_phentsize;
	mufs_file_seek(f, ehdr->e_phoff + index * size, SEEK_SET);
	if (mufs_file_read(f, (unsigned char *) phdr, size) != size) return EIO_ERR;
	return EOK;
}
