global load_tss
global jumpto_task
global jumpto_scheduler
global task_resume_yield
global task_resume_preempt
global task_resume_kernel_yield
global task_kernel_yield

extern task_running         ;; ukazatel na strukturu s aktualne bezicim procesem
extern task_scheduler       ;; funkce planovace procesu
extern task_kernel_switch   ;; funkce zajistujici prepnuti procesu beziciho v jadre

TSS_LOCATION equ 0x80000    ;; umisteni TSS  
KERNEL_STACK equ 0x50000    ;; zacatek jaderneho zasobniku
USER_STACK   equ 0x003f0000 ;; zacatek zasobniku v uzivatelskem prostoru

GDT_CS0      equ 0x08       ;; selector kodoveho segmentu jadra
GDT_DS0      equ 0x10       ;; selector dataveho segmentu jadra
GDT_TSS      equ 0x18       ;; selector TSS
GDT_CS3	     equ (0x20 | 3) ;; selector kodoveho segmentu uzivatelskeho prostoru
GDT_DS3	     equ (0x28 | 3) ;; selector kodoveho segmentu uzivatelskeho prostoru


;;
;; provede nastaveni registru vcetne zasobniku pro spusteni noveho procesu
;; a preda rizeni planovaci uloh
;;
jumpto_scheduler:
        mov ax, GDT_DS0
	mov ds, ax
        mov es, ax
        mov fs, ax
        mov gs, ax

	mov esp, KERNEL_STACK
	pushf
;	or dword [esp], 0x200 ; povoli preruseni

	push dword GDT_CS0
	push task_scheduler
	iret

;;
;; inicializuje TSS na nejnutnejsi minimum
;; nahraje selector TSS do registru TR
;;
load_tss:
	mov eax, TSS_LOCATION
	mov dword [eax + 4],   KERNEL_STACK
	mov word  [eax + 8],   GDT_DS0
	mov word  [eax + 102], 104 ; offset IOPB

	mov ax, GDT_TSS
	ltr ax
	ret

;;
;; provede prepnuti do ring3, vcetne nastaveni zasobniku
;; a zacne provadet kod od adresy ENTRY_POINT
;;
jumpto_task:
        mov ecx, [esp + 4]   ;; entry point
        mov ax, GDT_DS3
	mov ds, ax
        mov es, ax
        mov fs, ax
        mov gs, ax

        mov eax, USER_STACK
        push GDT_DS3
	push eax             ;; puvodni ESP (tj. stack uzivatelskeho prostoru)

	pushf
	or dword [esp], 0x200 ; povoli preruseni

        push GDT_CS3
	push ecx             ;; entry point
	iret

;;
;; Dobrovolne se vzda provadeni kodu v jadre a preda rizeni planovaci
;;
task_kernel_yield:
	push ebx                    ;; ulozi callee-saved registry (odpovida aktualnimu kontextu)
	push esi
	push edi

	mov eax, [task_running]     ;; ulozi ukazatel v zasobniku
	mov dword [eax + 4], esp

	call task_kernel_switch     ;; preda rizeni planovaci

        ; do teto casti by se uz nikdy nemel dostat

;;
;; Obnovi beh procesu, ktery se dobrovolne vzdalo procesoru pomoci sched_yield
;;
task_resume_yield:                  
        mov ax, GDT_DS3                 ;; obnovi segmentove registry
	mov ds, ax
        mov es, ax
        mov fs, ax
        mov gs, ax

	mov eax, [task_running]         ;; obnovi adresu v jadernem zasobniku
	mov esp, dword [eax + 4]

	add esp, 4                      ;; podle ulozenych hodnot obnovi stav registru
	pop ebx                         ;; hodnotu eax neobnovuje, umoznuje predat navratovou hodnotu
	pop ecx
	pop edx
	pop esi
	pop edi
	iret

;;
;; Obnovi beh procesu, ktery byl prerusen casovacem
;;
task_resume_preempt:
        mov ax, GDT_DS3                 ;; obnovi segmentove registry
	mov ds, ax
        mov es, ax
        mov fs, ax
        mov gs, ax

	mov eax, [task_running]         ;; obnovi adresu v jadernem zasobniku
	mov esp, dword [eax + 4]

	popad                           ;; obnoveni vsech registru a navrat z preruseni
	iret

;;
;; Obnovi beh procesu, ktery se dobrovolne vzdalo procesoru pri behu v jadre
;;
task_resume_kernel_yield:

	mov eax, [task_running]         ;; vrati adresu v jadernem zasobniku
	mov esp, dword [eax + 4]

	pop edi                         ;; obnovi stav registru
	pop esi
	pop ebx
	
	ret
