#ifndef TASKS_H
#define TASKS_H

#include "../fs/mufs.h"

#define MAX_PATH_LEN           (255)  /* maximalni velikost cesty */
#define MAX_FILES_OPENED	(20)  /* maximalni pocet popisovacu souboru */
#define MAX_ARGS                (20)  /* maximalni pocet argumentu programu */

#define TASK_NEW	         (1)  /* nove vytvoreny proces, ktery jeste nebezel */
#define TASK_READY	         (2)  /* proces pripraveny k behu */
#define TASK_RUNNING	         (3)  /* aktualne bezici proces */

#define ST_YIELD	         (1)  /* proces se dobrovolne vzdal procesoru */
#define ST_PREEMPT	         (2)  /* proces byl prerusen casovacem */
#define ST_KERNEL_YIELD          (3)  /* poces se dobrovolne vzdal procesoru, kdyz byl v jadre */


/**
 * Struktura popisujici aktualne bezici proces
 */
struct task {
	void *kernel_stack;                   /* adresa jaderneho zasobniku */
	void *esp;                         /* pozice v zasobniku */
	int last_err_code;                    /* navratovy kod, se kterym byla ukoncena posledni aplikace */
	unsigned long min_brk;                /* minimalni hranice oblasti s daty */
	unsigned long cur_brk;                /* aktualni hranice oblasti s daty */
	uint32_t *page_directory;             /* adresar tabulek stranek daneho procesu */
	struct mufs_fd *fd[MAX_FILES_OPENED]; /* descriptory otevrenych souboru */
	void (*entry_point)();                /* vstupni bod procesu */	
	char *path;                           /* cesta k souboru */
        char *argv[MAX_ARGS];                 /* pole s argumenty */
	int argc;                             /* pocet argumentu */
	char status;                          /* stav procesu */
	char switch_type;                     /* informace o typu prepnuti ulohy */
	int ticks;
};


void jumpto_task(unsigned long entry);
void jumpto_scheduler();

void task_resume_yield();
void task_resume_preempt();
void task_resume_kernel_yield();

void task_init();
void task_scheduler();
void task_exec(char *path, char **argv);
void task_terminate();
void task_kernel_yield();

#endif
